﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.IO;
using System.Security.Cryptography;

namespace TestWindow
{
    /// <summary>
    /// SecurityDrawLine.xaml の相互作用ロジック
    /// </summary>
    public partial class SecurityDrawLine : Window
    {
        /// <summary> 現在通過したポイントの数 </summary>
        private Int32 FLG_PASSPOINTNUM = 0;

        /// <summary> 通過したポイントの配列 </summary>
        private Int32[] PassPointArray = new Int32[9];

        /// <summary> Ellipseコントロールの配列 </summary>
        private Ellipse[] DOTARRAY = new Ellipse[10];

        /// <summary> OKライン </summary>
        //private Int32[] OKRoot = new int[9];
        /// <summary> OKライン(暗号化文字列) </summary>
        private String OKRootStr;

        /// <summary> OK時に表示するウィンドウ </summary>
        private Window PassedWindow;

        public SecurityDrawLine()
        {
            InitializeComponent();
        }

        /// <summary>
        /// ウィンドウをロードしたときの処理
        /// </summary>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            //配列化
            DOTARRAY[0] = DOT1;
            DOTARRAY[1] = DOT2;
            DOTARRAY[2] = DOT3;
            DOTARRAY[3] = DOT4;
            DOTARRAY[4] = DOT5;
            DOTARRAY[5] = DOT6;
            DOTARRAY[6] = DOT7;
            DOTARRAY[7] = DOT8;
            DOTARRAY[8] = DOT9;

            B_OK.IsEnabled = false;
        }


        /// <summary>
        /// OK時に表示するウィンドウを指定
        /// </summary>
        public void SetPassedWindow(Window dstwin)
        {
            PassedWindow = dstwin;
        }


        /// <summary>
        /// 正解ルート（暗号化文字列）をセット
        /// </summary>
        public void SetOKRootStr(String Str)
        {
            OKRootStr = Str;
        }


        /// <summary>
        /// ボタン：OK
        /// </summary>
        private void B_OK_Click(object sender, RoutedEventArgs e)
        {
            PassedWindow.Show();
            this.Close();
        }

        /// <summary>
        /// ボタン：閉じる
        /// </summary>
        private void B_CANCEL_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }


        /// <summary>
        /// マウスをクリックしたとき
        /// </summary>
        private void CV_MAIN_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            ResetDotStatus(1);

            Point pos = e.GetPosition((Canvas)sender);
            Int32 Target = DotTouchCheck(pos.X, pos.Y);
            Check_PassPoint(Target);
            Draw_PassPoint(pos.X, pos.Y, new SolidColorBrush(Color.FromRgb(0, 24, 240)));
        }


        /// <summary>
        /// マウスを動かしたとき
        /// </summary>
        private void CV_MAIN_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
            {
                Point pos = e.GetPosition((Canvas)sender);
                Int32 Target = DotTouchCheck(pos.X, pos.Y);
                Check_PassPoint(Target);

                Brush bru;

                if (FLG_PASSPOINTNUM < 9)
                {
                    bru = new SolidColorBrush(Color.FromRgb(0, 24, 240));
                }
                else
                {
                    if (Check_OKRoot() == true)
                    {
                        bru = new SolidColorBrush(Color.FromRgb(24, 240, 0));
                        B_OK.IsEnabled = true;
                    }
                    else
                    {
                        bru = new SolidColorBrush(Color.FromRgb(240, 24, 0));
                    }
                }

                Draw_PassPoint(pos.X, pos.Y, bru);
            }
            else
            {
                ResetDotStatus(0);
            }
        }

        /// <summary>
        /// マウスボタンを離したとき
        /// </summary>
        private void CV_MAIN_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            Point pos = e.GetPosition((Canvas)sender);
            Int32 Target = DotTouchCheck(pos.X, pos.Y);

            ResetDotStatus(0);
        }


        /// <summary>
        /// 触れているドット番号を取得する
        /// </summary>
        private Int32 DotTouchCheck(Double x, Double y)
        {
            Double size = DOTARRAY[0].Width;

            Int32 ret = -1;

            for (Int32 a = 0; a < 9; a++)
            {
                //DOT[X]と接触確認
                if ((x >= DOTARRAY[a].Margin.Left && x <= DOTARRAY[a].Margin.Left + size) &&
                    (y >= DOTARRAY[a].Margin.Top && y <= DOTARRAY[a].Margin.Top + size))
                {
                    ret = a;
                    return ret;
                }
            }

            return ret;
        }

        /// <summary>
        /// 通過ポイントをチェックする
        /// </summary>
        private void Check_PassPoint(Int32 target)
        {
            if (target == -1)
            {
                return;
            }

            if (FLG_PASSPOINTNUM == 0)
            {
                //最初の１点目の通過
                PassPointArray[0] = target;
                FLG_PASSPOINTNUM = 1;
            }
            else
            {
                //重複チェック
                Boolean dupCheck = false;
                for (Int32 a = 0; a < FLG_PASSPOINTNUM; a++)
                {
                    if (target == PassPointArray[a])
                    {
                        dupCheck = true;
                    }
                }

                if (dupCheck == false)
                {
                    PassPointArray[FLG_PASSPOINTNUM] = target;
                    FLG_PASSPOINTNUM++;
                }

            }
        }

        /// <summary>
        /// 通過ポイントを描画する
        /// </summary>
        private void Draw_PassPoint(Double x, Double y, Brush bru)
        {
            if(FLG_PASSPOINTNUM == 0)
            {
                CV_MAIN.Children.Clear();
                return;
            }

            //通過点を表示
            String statusStr = "通過[" + FLG_PASSPOINTNUM.ToString() + "点]：";
            for (Int32 a = 0; a < FLG_PASSPOINTNUM; a++)
            {
                statusStr += PassPointArray[a].ToString() + "→";
                DOTARRAY[PassPointArray[a]].Fill = bru;
            }

            L_DEBUG.Content = statusStr;


            //通過点の線を描画
            Double center = DOTARRAY[0].Width / 2;

            CV_MAIN.Children.Clear();

            for (Int32 a = 0; a < FLG_PASSPOINTNUM - 1; a++)
            {
                Line pointLine = new Line();
                pointLine.StrokeThickness = 10;
                pointLine.Stroke = bru;

                //点１
                pointLine.X1 = DOTARRAY[PassPointArray[a]].Margin.Left + center;
                pointLine.Y1 = DOTARRAY[PassPointArray[a]].Margin.Top + center;

                //点２
                pointLine.X2 = DOTARRAY[PassPointArray[a + 1]].Margin.Left + center;
                pointLine.Y2 = DOTARRAY[PassPointArray[a + 1]].Margin.Top + center;

                CV_MAIN.Children.Add(pointLine);
            }

            //通過点の楕円を描画
            for (Int32 a = 0; a < FLG_PASSPOINTNUM; a++)
            {
                Ellipse pointEllipse = new Ellipse();
                pointEllipse.StrokeThickness = 2;
                pointEllipse.Stroke = bru;

                pointEllipse.Width = 64;
                pointEllipse.Height = 64;

                pointEllipse.Margin = new Thickness(DOTARRAY[PassPointArray[a]].Margin.Left - 16, DOTARRAY[PassPointArray[a]].Margin.Top - 16, 0, 0);

                CV_MAIN.Children.Add(pointEllipse);
            }

            //最終通過点からドラッグ位置までの線を描画
            if (FLG_PASSPOINTNUM < 9)
            {
                Line pointLine2 = new Line();
                pointLine2.StrokeThickness = 10;
                pointLine2.Stroke = bru;

                //点１
                pointLine2.X1 = DOTARRAY[PassPointArray[FLG_PASSPOINTNUM-1]].Margin.Left + center;
                pointLine2.Y1 = DOTARRAY[PassPointArray[FLG_PASSPOINTNUM-1]].Margin.Top + center;

                //点２
                pointLine2.X2 = x;
                pointLine2.Y2 = y;

                CV_MAIN.Children.Add(pointLine2);
            }

        }

        /// <summary>
        /// 接触状態をクリアする
        /// </summary>
        private void ResetDotStatus(Int32 init)
        {
            //全点通過済みの場合は、判定処理へ
            if(FLG_PASSPOINTNUM == 9 && init == 0)
            {
                return;
            }

            FLG_PASSPOINTNUM = 0;
            B_OK.IsEnabled = false;

            for(Int32 a=0; a<9; a++)
            {
                DOTARRAY[a].Fill = new SolidColorBrush(Color.FromRgb(128, 128, 128));
            }

            CV_MAIN.Children.Clear();
        }


        /// <summary>
        /// OKルート判定
        /// </summary>
        private Boolean Check_OKRoot()
        {
            Boolean ret = false;

            //作成ルートを文字列化
            String checkStr = "";
            for(Int32 a=0; a<9; a++)
            {
                checkStr += PassPointArray[a].ToString();
            }

            String checkStr2 = StrEncrypt(checkStr);

            if(OKRootStr == checkStr2)
            {
                ret = true;
            }

            return ret;
        }


        /// <summary>
        /// 暗号化文字列を返す
        /// </summary>
        private String StrEncrypt(String SrcStr)
        {
            try
            {
                String textToEncrypt = SrcStr;
                String ret = "";

                string publickey = "12345678";
                string secretkey = "87654321";

                byte[] secretkeyByte = { };
                secretkeyByte = System.Text.Encoding.UTF8.GetBytes(secretkey);

                byte[] publickeybyte = { };
                publickeybyte = System.Text.Encoding.UTF8.GetBytes(publickey);

                MemoryStream ms = null;
                CryptoStream cs = null;

                byte[] inputbyteArray = System.Text.Encoding.UTF8.GetBytes(textToEncrypt);
                using (DESCryptoServiceProvider des = new DESCryptoServiceProvider())
                {
                    ms = new MemoryStream();
                    cs = new CryptoStream(ms, des.CreateEncryptor(publickeybyte, secretkeyByte), CryptoStreamMode.Write);
                    cs.Write(inputbyteArray, 0, inputbyteArray.Length);
                    cs.FlushFinalBlock();
                    ret = Convert.ToBase64String(ms.ToArray());
                }
                return ret;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message, ex.InnerException);
            }
        }


        /// <summary>
        /// 暗号化された正解ルートの文字列を取得
        /// </summary>
        /// <returns></returns>
        public String GetOKRootStr(String SrcStr)
        {
            return StrEncrypt(SrcStr);
        }


    }
}
